;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-

;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          api.lisp
;;;; Purpose:       cl-youtube Common Lisp API
;;;; Programmer:    Nicolas Lamirault <nicolas.lamirault@gmail.com>
;;;;
;;;; This file, part of cl-youtube, is Copyright (c) 2007 by Nicolas Lamirault
;;;;
;;;; cl-youtube users are granted the rights to distribute and use this software
;;;; as governed by the terms of the Lisp Lesser GNU Public License
;;;; (http://opensource.franz.com/preamble.html), also known as the LLGPL.
;;;;
;;;; *************************************************************************


(in-package :cl-youtube)



(defclass client ()
  ((id :initform nil
       :initarg :id
       :accessor youtube-id
       :documentation "YouTube developer ID."))
  (:documentation "A YouTube developper profile."))


(defun perform-request (request)
  "Perform the REST request, and return the body of the response."
  (multiple-value-bind (body-stream status-code headers uri stream must-close)
        (drakma:http-request request)
      (declare (ignore headers uri stream must-close))
      (when (and status-code (= status-code 200))
        body-stream)))



;; ------------------
;; User informations
;; ------------------

(defgeneric get-user-profile (client name)
  (:documentation "Retrieves the public parts of a user profile."))


(defmethod get-user-profile ((client client) name)
  (with-slots (id) client
    (let ((request (format nil *rest-user-profile* id name)))
      (format t "~A~%" (perform-request request)))))


(defgeneric get-user-favorite-videos (client name)
  (:documentation "Lists a user's favorite videos."))


(defmethod get-user-favorite-videos ((client client) name)
  (with-slots (id) client
    (let ((request (format nil *rest-user-videos* id name)))
      (format t "~A~%" (perform-request request)))))


(defgeneric get-user-friends (client name)
  (:documentation "Lists a user's friends."))


(defmethod get-user-friends ((client client) name)
  (with-slots (id) client
    (let ((request (format nil *rest-user-friends* id name)))
      (format t "~A~%" (perform-request request)))))


;; ---------------
;; Videos Viewing
;; ---------------



(defgeneric get-video-details (client video-id)
  (:documentation "Displays the details for a video."))


(defmethod get-video-details ((client client) video-id)
  (with-slots (id) client
    (let ((request (format nil *rest-video-details* id video-id)))
      (format t "~A~%" (perform-request request)))))


(defgeneric get-tag-videos (client tag &key page per-page)
  (:documentation "Lists all videos that have the specified tag."))


(defmethod get-tag-videos ((client client) tag &key page per-page)
  (declare (ignore page per-page))
  (with-slots (id) client
    (let ((request (format nil *rest-tag-videos* id tag)))
      (format t "~A~%" (perform-request request)))))


(defgeneric get-user-videos (client name)
  (:documentation "Lists all videos that were uploaded by the specified user."))


(defmethod get-user-videos ((client client) name)
  (with-slots (id) client
    (let ((request (format nil *rest-user-videos* id name)))
      (format t "~A~%" (perform-request request)))))


(defgeneric get-videos-featured (client)
  (:documentation " Lists the most recent 25 videos that have been featured on
the front page of the YouTube site."))


(defmethod get-videos-featured ((client client))
  (with-slots (id) client
    (let ((request (format nil *rest-videos-featured* id)))
      (format t "~A~%" (perform-request request)))))